@@ -288,8 +288,8 @@ int intel_dp_mtp_tu_compute_config(struct intel_dp *intel_dp,
 			return PTR_ERR(mst_state);
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(6, 14, 0)
-                mst_state->pbn_div = drm_dp_get_vc_payload_bw(&intel_dp->mst.mgr,
-                                                              crtc_state->port_clock,
+		mst_state->pbn_div = drm_dp_get_vc_payload_bw(&intel_dp->mst.mgr,
+							      crtc_state->port_clock,
 							      crtc_state->lane_count);
 #else
 		mst_state->pbn_div = drm_dp_get_vc_payload_bw(crtc_state->port_clock,
@@ -617,12 +617,15 @@ adjust_limits_for_dsc_hblank_expansion_quirk(struct intel_dp *intel_dp,
 
 static bool
 mst_stream_compute_config_limits(struct intel_dp *intel_dp,
-				 struct intel_connector *connector,
+				 struct drm_connector_state *conn_state,
 				 struct intel_crtc_state *crtc_state,
 				 bool dsc,
 				 struct link_config_limits *limits)
 {
-	if (!intel_dp_compute_config_limits(intel_dp, connector,
+	struct intel_connector *connector =
+		to_intel_connector(conn_state->connector);
+
+	if (!intel_dp_compute_config_limits(intel_dp, conn_state,
 					    crtc_state, false, dsc,
 					    limits))
 		return false;
@@ -671,7 +674,7 @@ static int mst_stream_compute_config(struct intel_encoder *encoder,
 	joiner_needs_dsc = intel_dp_joiner_needs_dsc(display, num_joined_pipes);
 
 	dsc_needed = joiner_needs_dsc || intel_dp->force_dsc_en ||
-		!mst_stream_compute_config_limits(intel_dp, connector,
+		!mst_stream_compute_config_limits(intel_dp, conn_state,
 						  pipe_config, false, &limits);
 
 	if (!dsc_needed) {
@@ -697,7 +700,7 @@ static int mst_stream_compute_config(struct intel_encoder *encoder,
 			    str_yes_no(intel_dp->force_dsc_en));
 
 
-		if (!mst_stream_compute_config_limits(intel_dp, connector,
+		if (!mst_stream_compute_config_limits(intel_dp, conn_state,
 						      pipe_config, true,
 						      &limits))
 			return -EINVAL;
@@ -1756,7 +1759,7 @@ mst_topology_add_connector(struct drm_dp_mst_topology_mgr *mgr,
 #endif
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(6, 14, 0)
-        ret = drm_connector_init(display->drm, &connector->base, &mst_connector_funcs,
+	ret = drm_connector_init(display->drm, &connector->base, &mst_connector_funcs,
 					 DRM_MODE_CONNECTOR_DisplayPort);
 #else
 	ret = drm_connector_dynamic_init(display->drm, &connector->base, &mst_connector_funcs,
