@@ -9,4 +9,10 @@ int ttm_bo_access(struct ttm_buffer_object *bo, unsigned long offset,
 		  void *buf, int len, int write);
 #endif
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(6, 18, 0)
+#define ttm_bo_setup_export LINUX_BACKPORT(ttm_bo_setup_export)
+int ttm_bo_setup_export(struct ttm_buffer_object *bo,
+			struct ttm_operation_ctx *ctx);
+#endif
+
 #endif
